from functools import wraps
from errors import ParameterError

def check_required(f):
  @wraps(f)
  def wrapper(instance, data):
    required = globals()[f.__name__]()
    for param in required:
      if param not in data:
        raise ParameterError(param)
    return f(instance, data)

  return wrapper

def optimize():
  return ['file_url', 'file_name', 'geocode', 'stop_name', 'date_starting',
    'warehouse', 'num_drivers', 'num_avg_service_time', 'float_fuel_cost',
    'float_driver_wage', 'float_mpg']



from setuptools import setup

VERSION = '0.0.3'

def readme():
  with open('README.md') as f:
    return f.read()

setup(
  name = 'foxtrot-client',
  packages = ['foxtrot'],
  version = VERSION,
  description = 'Foxtrot Client Library',
  long_description = readme(),
  author = 'Yasyf Mohamedali',
  author_email = 'yasyf@foxtrot.io',
  url = 'https://github.com/FoxtrotSystems/api-client-python',
  download_url = 'https://github.com/FoxtrotSystems/api-client-python/tarball/' + VERSION,
  license = 'MIT',
  keywords = ['foxtrot', 'route optimization'],
  install_requires = ['requests']
)



import requests, json
from urllib import urlencode
from _required_fields import check_required
from response import FoxtrotResponse

class Foxtrot(object):
  """Foxtrot API Client"""

  host = 'app.foxtrot.io'
  ssl = True

  def __init__(self, api_key, version='v1'):
    self.api_key = api_key
    self.version = version
    self.headers = {'content-type': 'application/json'}

  def __build_url(self, endpoint, args):
    proto = "https" if self.ssl else "http"
    args['key'] = self.api_key
    query = urlencode(args)
    return "{}://{}/api/{}/{}?{}".format(proto, self.host, self.version, endpoint, query)

  def __get_request(self, endpoint, args):
    url = self.__build_url(endpoint, args)
    response = requests.get(url, headers=self.headers)
    return response.json()

  def __post_request(self, endpoint, data, args):
    url = self.__build_url(endpoint, args)
    response = requests.post(url, data=json.dumps(data), headers=self.headers)
    return response.json()

  def _poll(self, endpoint, txid, args={}):
    resp = self.__get_request(endpoint, {'txid': txid})
    return FoxtrotResponse.response_for(endpoint, resp, self)

  @check_required
  def optimize(self, data, args={}):
    resp = self.__post_request('optimize', data, args)
    return FoxtrotResponse.response_for('optimize', resp, self)
